@@ -8,10 +8,11 @@
 //!   it’s possible to specify the number of vertices to render or just let the [`Tess`] render
 //!   a default number of vertices (typically, the whole [`Tess`]).
 //! - A default number of _instances_, which allows for geometry instancing. Geometry instancing
-//!   is the fact of drawing with the same [`Tess`] (GPU buffers) several times, only changing the
-//!   instance index every time a new render is performed. This is done entirely on the GPU to
+//!   is the fact of drawing with the same [`Tess`] several times, only changing the
+//!   instance index every time a new render is performed. This is done entirely on the backend to
 //!   prevent bandwidth exhaustion. The index of the instance, in the shader stages, is often used
-//!   to pick material properties, matrices, etc. to customize each instances.
+//!   to pick material properties, matrices, etc. to customize each instances. Instances can manually
+//!   be asked when using a [`TessView`].
 //! - An indexed configuration, allowing to tell the GPU how to render the vertices by referring to
 //!   them via indices.
 //! - For indexed configuration, an optional _primitive restart index_ can be specified. That
@@ -38,14 +39,15 @@
 //! [`TessGate`]. In order to do so, you need a [`TessView`].
 //!
 //! A [`TessView`] is a temporary _view_ into a [`Tess`], describing what part of it should be
-//! drawn. Creating [`TessView`]s is a cheap operation, and can be done in two different ways:
+//! drawn. It is also responsible in providing the number of instances to draw.
+//! Creating [`TessView`]s is a cheap operation, and can be done in two different ways:
 //!
 //! - By directly using the methods from [`TessView`].
 //! - By using the [`View`] trait.
 //!
 //! The [`View`] trait is a convenient way to create [`TessView`]. It provides the
-//! [`View::view`] and [`View::inst_view`] methods, which accept Rust’s range operators
-//! to create the [`TessView`]s in a more comfortable way.
+//! [`View::view`] and [`View::inst_view`] (for instanced rendering) methods, which accept Rust’s
+//! range operators to create the [`TessView`]s in a more comfortable way.
 //!
 //! # Tessellation mapping
 //!
@@ -69,7 +71,6 @@ use crate::{
     IndexSlice as IndexSliceBackend, InstanceSlice as InstanceSliceBackend, Tess as TessBackend,
     VertexSlice as VertexSliceBackend,
   },
-  buffer::BufferError,
   context::GraphicsContext,
   vertex::{Deinterleave, Vertex, VertexDesc},
 };
@@ -79,7 +80,7 @@ use std::{
   ops::{Deref, DerefMut, Range, RangeFrom, RangeFull, RangeInclusive, RangeTo, RangeToInclusive},
 };
 
-/// Vertices can be connected via several modes.
+/// Primitive mode.
 ///
 /// Some modes allow for _primitive restart_. Primitive restart is a cool feature that allows to
 /// _break_ the building of a primitive to _start over again_. For instance, when making a curve,
@@ -92,6 +93,9 @@ use std::{
 /// _Primitive restart_ should be used as much as possible as it will decrease the number of GPU
 /// commands you have to issue.
 ///
+/// > Deprecation notice: the next version of luminance will not support setting the primitive restart index: you will
+/// then must provide the maximum value of index type.
+///
 /// That feature is encoded with a special _vertex index_. You can setup the value of the _primitive
 /// restart index_ with [`TessBuilder::set_primitive_restart_index`]. Whenever a vertex index is set
 /// to the same value as the _primitive restart index_, the value is not interpreted as a vertex
@@ -166,8 +170,8 @@ impl fmt::Display for Mode {
 #[non_exhaustive]
 #[derive(Debug, Eq, PartialEq)]
 pub enum TessMapError {
-  /// The CPU mapping failed due to buffer errors.
-  BufferMapError(BufferError),
+  /// Cannot obtain a slice on the backend.
+  CannotMap,
   /// Vertex target type is not the same as the one stored in the buffer.
   VertexTypeMismatch(VertexDesc, VertexDesc),
   /// Index target type is not the same as the one stored in the buffer.
@@ -181,9 +185,9 @@ pub enum TessMapError {
 }
 
 impl TessMapError {
-  /// The CPU mapping failed due to buffer errors.
-  pub fn buffer_map_error(e: BufferError) -> Self {
-    TessMapError::BufferMapError(e)
+  /// Cannot obtain a slice on the backend.
+  pub fn cannot_map() -> Self {
+    TessMapError::CannotMap
   }
 
   /// Vertex target type is not the same as the one stored in the buffer.
@@ -212,20 +216,24 @@ impl TessMapError {
 impl fmt::Display for TessMapError {
   fn fmt(&self, f: &mut fmt::Formatter) -> Result<(), fmt::Error> {
     match *self {
-      TessMapError::BufferMapError(ref e) => write!(f, "cannot map tessellation buffer: {}", e),
+      TessMapError::CannotMap => f.write_str("cannot map on the backend"),
+
       TessMapError::VertexTypeMismatch(ref a, ref b) => write!(
         f,
         "cannot map tessellation: vertex type mismatch between {:?} and {:?}",
         a, b
       ),
+
       TessMapError::IndexTypeMismatch(ref a, ref b) => write!(
         f,
         "cannot map tessellation: index type mismatch between {:?} and {:?}",
         a, b
       ),
+
       TessMapError::ForbiddenAttributelessMapping => {
         f.write_str("cannot map an attributeless buffer")
       }
+
       TessMapError::ForbiddenDeinterleavedMapping => {
         f.write_str("cannot map a deinterleaved buffer as interleaved")
       }
@@ -233,20 +241,7 @@ impl fmt::Display for TessMapError {
   }
 }
 
-impl From<BufferError> for TessMapError {
-  fn from(e: BufferError) -> Self {
-    TessMapError::buffer_map_error(e)
-  }
-}
-
-impl error::Error for TessMapError {
-  fn source(&self) -> Option<&(dyn error::Error + 'static)> {
-    match self {
-      TessMapError::BufferMapError(e) => Some(e),
-      _ => None,
-    }
-  }
-}
+impl error::Error for TessMapError {}
 
 /// Possible errors that might occur when dealing with [`Tess`].
 #[non_exhaustive]
@@ -258,8 +253,6 @@ pub enum TessError {
   AttributelessError(String),
   /// Length incoherency in vertex, index or instance buffers.
   LengthIncoherency(usize),
-  /// Internal error ocurring with a buffer.
-  InternalBufferError(BufferError),
   /// Forbidden primitive mode by hardware.
   ForbiddenPrimitiveMode(Mode),
   /// No data provided and empty tessellation.
@@ -282,11 +275,6 @@ impl TessError {
     TessError::LengthIncoherency(len)
   }
 
-  /// Internal error ocurring with a buffer.
-  pub fn internal_buffer_error(e: BufferError) -> Self {
-    TessError::InternalBufferError(e)
-  }
-
   /// Forbidden primitive mode by hardware.
   pub fn forbidden_primitive_mode(mode: Mode) -> Self {
     TessError::ForbiddenPrimitiveMode(mode)
@@ -306,27 +294,13 @@ impl fmt::Display for TessError {
       TessError::LengthIncoherency(ref s) => {
         write!(f, "Incoherent size for internal buffers: {}", s)
       }
-      TessError::InternalBufferError(ref e) => write!(f, "internal buffer error: {}", e),
       TessError::ForbiddenPrimitiveMode(ref e) => write!(f, "forbidden primitive mode: {}", e),
       TessError::NoData => f.write_str("no data or empty tessellation"),
     }
   }
 }
 
-impl From<BufferError> for TessError {
-  fn from(e: BufferError) -> Self {
-    TessError::internal_buffer_error(e)
-  }
-}
-
-impl error::Error for TessError {
-  fn source(&self) -> Option<&(dyn error::Error + 'static)> {
-    match self {
-      TessError::InternalBufferError(e) => Some(e),
-      _ => None,
-    }
-  }
-}
+impl error::Error for TessError {}
 
 /// Possible tessellation index types.
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
@@ -416,6 +390,28 @@ pub enum Interleaved {}
 pub enum Deinterleaved {}
 
 /// Vertex input data of a [`TessBuilder`].
+///
+/// This trait defines the _storage_ of vertices that a [`TessBuilder`] will use to build its internal storage on the
+/// backend.
+///
+/// There are two implementors of this trait:
+///
+/// - `impl<V> TessVertexData<Interleaved> for V where V: Vertex`
+/// - `impl<V> TessVertexData<Deinterleaved> for V where V: Vertex`
+///
+/// For the situation where `S` is [`Interleaved`], this trait associates the data (with [`TessVertexData::Data`]) to be
+/// a `Vec<V>`. What it means is that the [`TessBuilder`] will build the vertices as a `Vec<V>`, where `V: Vertex`,
+/// implementing an _interleaved memory layout_.
+///
+/// For the situation where `S` is [`Deinterleaved`], this trait associates the data to be a `Vec<DeinterleavedData>`.
+/// [`DeinterleavedData`] is a special type used to store a collection of one of the attributes of a `V: Vertex`. For
+/// instance, if `V: Vertex` has two attributes, vertices will end up in two [`DeinterleavedData`]: the first one for
+/// the first attribute, the second one for the second attribute. The [`TessBuilder`] will handle that logic for you
+/// when you will use the [`TessBuilder::set_vertices`] by tracking at the type-level which set of attributes you are setting.
+///
+/// # Parametricity
+///
+/// - `S` is the storage marker. It will be set to either [`Interleaved`] or [`Deinterleaved`].
 pub trait TessVertexData<S>: Vertex
 where
   S: ?Sized,
@@ -425,8 +421,8 @@ where
 
   /// Coherent length of the vertices.
   ///
-  /// Vertices length can be incohent for some implementations of [`TessVertexData::Data`],
-  /// especially with deinterleaved memory.
+  /// Vertices length can be incoherent for some implementations of [`TessVertexData::Data`],
+  /// especially with deinterleaved memory. For this reason, this method can fail with [`TessError`].
   fn coherent_len(data: &Self::Data) -> Result<usize, TessError>;
 }
 
@@ -463,6 +459,9 @@ where
 }
 
 /// Deinterleaved data.
+///
+/// [`DeinterleavedData`] represents a collection of one type of attributes of a set of vertices, for each vertex
+/// implements [`Vertex`]. End-users shouldn’t need to know about this type as it’s only used internally.
 #[derive(Debug, Clone)]
 pub struct DeinterleavedData {
   raw: Vec<u8>,
@@ -500,7 +499,7 @@ impl DeinterleavedData {
 ///
 /// This is the same as interleaved data in terms of interface, but the `T` type is interpreted
 /// a bit differently. Here, the encoding is `(Vec<Field0>, Vec<Field1>, …)`, where `Field0`,
-/// `Field1` etc. are all the ordered fieds in `T`.
+/// `Field1` etc. are all the ordered fieds in `T`. This logic is hidden behind `Vec<DeinterleavedData>`.
 ///
 /// That representation allows field-based operations on [`Tess`], while it would be impossible
 /// with the interleaved version (you would need to get all the fields at once, since
@@ -509,10 +508,8 @@ impl DeinterleavedData {
 /// # Data encoding
 ///
 /// - Vectors: you can pass vectors as input data for both vertices and indices. Those will be
-///   interpreted differently based on the vertex storage you chose for vertices, and the normal
-///   way for indices.
-/// - Buffers: you can pass [`Buffer`] objects, too. Those are more flexible than vectors as you can
-///   use all of the [`Buffer`] API before sending them to the builder.
+///   interpreted differently based on the vertex storage you chose for vertices / instances. For indices, there is no
+///   difference.
 /// - Disabled: disabling means that no data will be passed to the GPU. You can disable independently
 ///   vertex data and/or index data by using the unit `()` type.
 ///
@@ -522,9 +519,7 @@ impl DeinterleavedData {
 /// regions of memory (`Vec<T>`), where `T` satisfies [`TessIndex`]. When using an indexed tessellation,
 /// the meaning of its attributes slightly changes. First, the vertices are not used as input source for
 /// the vertex stream. In order to provide vertices that will go through the vertex stream, the indices
-/// reference the vertex set to provide the order in which they should appear in the stream. That has a
-/// consequence on the meaning of subsequent operations, such as [`Tess::vert_nb`] and how rendering
-/// works.
+/// reference the vertex set to provide the order in which they should appear in the stream.
 ///
 /// When rendering with a [`TessView`], the number of vertices to render must be provided or inferred
 /// based on the [`Tess`] the view was made from. That number will refer to either the vertex set or
@@ -532,6 +527,12 @@ impl DeinterleavedData {
 /// pick 3 vertices from the vertex set for direct tessellations and 3 indices to index the vertex set
 /// for indexed tessellations.
 ///
+/// # Primitive mode
+///
+/// By default, a [`TessBuilder`] will build _points_. Each vertex in the vertex stream will be independently rendered
+/// from the others, resulting in a _point cloud_. This logic is encoded with [`Mode::Point`]. You can change how
+/// vertices are interpreted by changing the [`Mode`].
+///
 /// # Parametricity
 ///
 /// - `B` is the backend type
@@ -539,8 +540,6 @@ impl DeinterleavedData {
 /// - `I` is the index type.
 /// - `W` is the vertex instance type.
 /// - `S` is the storage type.
-///
-/// [`Buffer`]: crate::buffer::Buffer
 #[derive(Debug)]
 pub struct TessBuilder<'a, B, V, I = (), W = (), S = Interleaved>
 where
@@ -986,9 +985,9 @@ where
   /// Slice the [`Tess`] in order to read its content via usual slices.
   ///
   /// This method gives access to the underlying _index storage_.
-  pub fn indices(&mut self) -> Result<Indices<B, V, I, W, S>, TessMapError>
+  pub fn indices<'a>(&'a mut self) -> Result<Indices<'a, B, V, I, W, S>, TessMapError>
   where
-    B: IndexSliceBackend<V, I, W, S>,
+    B: IndexSliceBackend<'a, V, I, W, S>,
   {
     unsafe { B::indices(&mut self.repr).map(|repr| Indices { repr }) }
   }
@@ -996,9 +995,9 @@ where
   /// Slice the [`Tess`] in order to read its content via usual slices.
   ///
   /// This method gives access to the underlying _index storage_.
-  pub fn indices_mut(&mut self) -> Result<IndicesMut<B, V, I, W, S>, TessMapError>
+  pub fn indices_mut<'a>(&'a mut self) -> Result<IndicesMut<'a, B, V, I, W, S>, TessMapError>
   where
-    B: IndexSliceBackend<V, I, W, S>,
+    B: IndexSliceBackend<'a, V, I, W, S>,
   {
     unsafe { B::indices_mut(&mut self.repr).map(|repr| IndicesMut { repr }) }
   }
@@ -1014,9 +1013,11 @@ where
   /// Slice the [`Tess`] in order to read its content via usual slices.
   ///
   /// This method gives access to the underlying _vertex storage_.
-  pub fn vertices(&mut self) -> Result<Vertices<B, V, I, W, Interleaved, V>, TessMapError>
+  pub fn vertices<'a>(
+    &'a mut self,
+  ) -> Result<Vertices<'a, B, V, I, W, Interleaved, V>, TessMapError>
   where
-    B: VertexSliceBackend<V, I, W, Interleaved, V>,
+    B: VertexSliceBackend<'a, V, I, W, Interleaved, V>,
   {
     unsafe { B::vertices(&mut self.repr).map(|repr| Vertices { repr }) }
   }
@@ -1024,9 +1025,11 @@ where
   /// Slice the [`Tess`] in order to read its content via usual slices.
   ///
   /// This method gives access to the underlying _vertex storage_.
-  pub fn vertices_mut(&mut self) -> Result<VerticesMut<B, V, I, W, Interleaved, V>, TessMapError>
+  pub fn vertices_mut<'a>(
+    &'a mut self,
+  ) -> Result<VerticesMut<'a, B, V, I, W, Interleaved, V>, TessMapError>
   where
-    B: VertexSliceBackend<V, I, W, Interleaved, V>,
+    B: VertexSliceBackend<'a, V, I, W, Interleaved, V>,
   {
     unsafe { B::vertices_mut(&mut self.repr).map(|repr| VerticesMut { repr }) }
   }
@@ -1034,9 +1037,11 @@ where
   /// Slice the [`Tess`] in order to read its content via usual slices.
   ///
   /// This method gives access to the underlying _instance storage_.
-  pub fn instances(&mut self) -> Result<Instances<B, V, I, W, Interleaved, W>, TessMapError>
+  pub fn instances<'a>(
+    &'a mut self,
+  ) -> Result<Instances<'a, B, V, I, W, Interleaved, W>, TessMapError>
   where
-    B: InstanceSliceBackend<V, I, W, Interleaved, W>,
+    B: InstanceSliceBackend<'a, V, I, W, Interleaved, W>,
   {
     unsafe { B::instances(&mut self.repr).map(|repr| Instances { repr }) }
   }
@@ -1044,9 +1049,11 @@ where
   /// Slice the [`Tess`] in order to read its content via usual slices.
   ///
   /// This method gives access to the underlying _instance storage_.
-  pub fn instances_mut(&mut self) -> Result<InstancesMut<B, V, I, W, Interleaved, W>, TessMapError>
+  pub fn instances_mut<'a>(
+    &'a mut self,
+  ) -> Result<InstancesMut<'a, B, V, I, W, Interleaved, W>, TessMapError>
   where
-    B: InstanceSliceBackend<V, I, W, Interleaved, W>,
+    B: InstanceSliceBackend<'a, V, I, W, Interleaved, W>,
   {
     unsafe { B::instances_mut(&mut self.repr).map(|repr| InstancesMut { repr }) }
   }
@@ -1062,9 +1069,11 @@ where
   /// Slice the [`Tess`] in order to read its content via usual slices.
   ///
   /// This method gives access to the underlying _vertex storage_.
-  pub fn vertices<T>(&mut self) -> Result<Vertices<B, V, I, W, Deinterleaved, T>, TessMapError>
+  pub fn vertices<'a, T>(
+    &'a mut self,
+  ) -> Result<Vertices<'a, B, V, I, W, Deinterleaved, T>, TessMapError>
   where
-    B: VertexSliceBackend<V, I, W, Deinterleaved, T>,
+    B: VertexSliceBackend<'a, V, I, W, Deinterleaved, T>,
     V: Deinterleave<T>,
   {
     unsafe { B::vertices(&mut self.repr).map(|repr| Vertices { repr }) }
@@ -1073,11 +1082,11 @@ where
   /// Slice the [`Tess`] in order to read its content via usual slices.
   ///
   /// This method gives access to the underlying _vertex storage_.
-  pub fn vertices_mut<T>(
-    &mut self,
-  ) -> Result<VerticesMut<B, V, I, W, Deinterleaved, T>, TessMapError>
+  pub fn vertices_mut<'a, T>(
+    &'a mut self,
+  ) -> Result<VerticesMut<'a, B, V, I, W, Deinterleaved, T>, TessMapError>
   where
-    B: VertexSliceBackend<V, I, W, Deinterleaved, T>,
+    B: VertexSliceBackend<'a, V, I, W, Deinterleaved, T>,
     V: Deinterleave<T>,
   {
     unsafe { B::vertices_mut(&mut self.repr).map(|repr| VerticesMut { repr }) }
@@ -1086,9 +1095,11 @@ where
   /// Slice the [`Tess`] in order to read its content via usual slices.
   ///
   /// This method gives access to the underlying _instance storage_.
-  pub fn instances<T>(&mut self) -> Result<Instances<B, V, I, W, Deinterleaved, T>, TessMapError>
+  pub fn instances<'a, T>(
+    &'a mut self,
+  ) -> Result<Instances<'a, B, V, I, W, Deinterleaved, T>, TessMapError>
   where
-    B: InstanceSliceBackend<V, I, W, Deinterleaved, T>,
+    B: InstanceSliceBackend<'a, V, I, W, Deinterleaved, T>,
     W: Deinterleave<T>,
   {
     unsafe { B::instances(&mut self.repr).map(|repr| Instances { repr }) }
@@ -1097,11 +1108,11 @@ where
   /// Slice the [`Tess`] in order to read its content via usual slices.
   ///
   /// This method gives access to the underlying _instance storage_.
-  pub fn instances_mut<T>(
-    &mut self,
-  ) -> Result<InstancesMut<B, V, I, W, Deinterleaved, T>, TessMapError>
+  pub fn instances_mut<'a, T>(
+    &'a mut self,
+  ) -> Result<InstancesMut<'a, B, V, I, W, Deinterleaved, T>, TessMapError>
   where
-    B: InstanceSliceBackend<V, I, W, Deinterleaved, T>,
+    B: InstanceSliceBackend<'a, V, I, W, Deinterleaved, T>,
     W: Deinterleave<T>,
   {
     unsafe { B::instances_mut(&mut self.repr).map(|repr| InstancesMut { repr }) }
@@ -1110,9 +1121,9 @@ where
 
 /// TODO
 #[derive(Debug)]
-pub struct Vertices<B, V, I, W, S, T>
+pub struct Vertices<'a, B, V, I, W, S, T>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + VertexSliceBackend<V, I, W, S, T>,
+  B: ?Sized + TessBackend<V, I, W, S> + VertexSliceBackend<'a, V, I, W, S, T>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1121,9 +1132,9 @@ where
   repr: B::VertexSliceRepr,
 }
 
-impl<B, V, I, W, S, T> Deref for Vertices<B, V, I, W, S, T>
+impl<'a, B, V, I, W, S, T> Deref for Vertices<'a, B, V, I, W, S, T>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + VertexSliceBackend<V, I, W, S, T>,
+  B: ?Sized + TessBackend<V, I, W, S> + VertexSliceBackend<'a, V, I, W, S, T>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1138,9 +1149,9 @@ where
 
 /// TODO
 #[derive(Debug)]
-pub struct VerticesMut<B, V, I, W, S, T>
+pub struct VerticesMut<'a, B, V, I, W, S, T>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + VertexSliceBackend<V, I, W, S, T>,
+  B: ?Sized + TessBackend<V, I, W, S> + VertexSliceBackend<'a, V, I, W, S, T>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1149,9 +1160,9 @@ where
   repr: B::VertexSliceMutRepr,
 }
 
-impl<B, V, I, W, S, T> Deref for VerticesMut<B, V, I, W, S, T>
+impl<'a, B, V, I, W, S, T> Deref for VerticesMut<'a, B, V, I, W, S, T>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + VertexSliceBackend<V, I, W, S, T>,
+  B: ?Sized + TessBackend<V, I, W, S> + VertexSliceBackend<'a, V, I, W, S, T>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1164,9 +1175,9 @@ where
   }
 }
 
-impl<B, V, I, W, S, T> DerefMut for VerticesMut<B, V, I, W, S, T>
+impl<'a, B, V, I, W, S, T> DerefMut for VerticesMut<'a, B, V, I, W, S, T>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + VertexSliceBackend<V, I, W, S, T>,
+  B: ?Sized + TessBackend<V, I, W, S> + VertexSliceBackend<'a, V, I, W, S, T>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1179,9 +1190,9 @@ where
 
 /// TODO
 #[derive(Debug)]
-pub struct Indices<B, V, I, W, S>
+pub struct Indices<'a, B, V, I, W, S>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + IndexSliceBackend<V, I, W, S>,
+  B: ?Sized + TessBackend<V, I, W, S> + IndexSliceBackend<'a, V, I, W, S>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1190,9 +1201,9 @@ where
   repr: B::IndexSliceRepr,
 }
 
-impl<B, V, I, W, S> Deref for Indices<B, V, I, W, S>
+impl<'a, B, V, I, W, S> Deref for Indices<'a, B, V, I, W, S>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + IndexSliceBackend<V, I, W, S>,
+  B: ?Sized + TessBackend<V, I, W, S> + IndexSliceBackend<'a, V, I, W, S>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1207,9 +1218,9 @@ where
 
 /// TODO
 #[derive(Debug)]
-pub struct IndicesMut<B, V, I, W, S>
+pub struct IndicesMut<'a, B, V, I, W, S>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + IndexSliceBackend<V, I, W, S>,
+  B: ?Sized + TessBackend<V, I, W, S> + IndexSliceBackend<'a, V, I, W, S>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1218,9 +1229,9 @@ where
   repr: B::IndexSliceMutRepr,
 }
 
-impl<B, V, I, W, S> Deref for IndicesMut<B, V, I, W, S>
+impl<'a, B, V, I, W, S> Deref for IndicesMut<'a, B, V, I, W, S>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + IndexSliceBackend<V, I, W, S>,
+  B: ?Sized + TessBackend<V, I, W, S> + IndexSliceBackend<'a, V, I, W, S>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1233,9 +1244,9 @@ where
   }
 }
 
-impl<B, V, I, W, S> DerefMut for IndicesMut<B, V, I, W, S>
+impl<'a, B, V, I, W, S> DerefMut for IndicesMut<'a, B, V, I, W, S>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + IndexSliceBackend<V, I, W, S>,
+  B: ?Sized + TessBackend<V, I, W, S> + IndexSliceBackend<'a, V, I, W, S>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1248,9 +1259,9 @@ where
 
 /// TODO
 #[derive(Debug)]
-pub struct Instances<B, V, I, W, S, T>
+pub struct Instances<'a, B, V, I, W, S, T>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + InstanceSliceBackend<V, I, W, S, T>,
+  B: ?Sized + TessBackend<V, I, W, S> + InstanceSliceBackend<'a, V, I, W, S, T>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1259,9 +1270,9 @@ where
   repr: B::InstanceSliceRepr,
 }
 
-impl<B, V, I, W, S, T> Deref for Instances<B, V, I, W, S, T>
+impl<'a, B, V, I, W, S, T> Deref for Instances<'a, B, V, I, W, S, T>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + InstanceSliceBackend<V, I, W, S, T>,
+  B: ?Sized + TessBackend<V, I, W, S> + InstanceSliceBackend<'a, V, I, W, S, T>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1276,9 +1287,9 @@ where
 
 /// TODO
 #[derive(Debug)]
-pub struct InstancesMut<B, V, I, W, S, T>
+pub struct InstancesMut<'a, B, V, I, W, S, T>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + InstanceSliceBackend<V, I, W, S, T>,
+  B: ?Sized + TessBackend<V, I, W, S> + InstanceSliceBackend<'a, V, I, W, S, T>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
@@ -1287,9 +1298,9 @@ where
   repr: B::InstanceSliceMutRepr,
 }
 
-impl<B, V, I, W, S, T> Deref for InstancesMut<B, V, I, W, S, T>
+impl<'a, B, V, I, W, S, T> Deref for InstancesMut<'a, B, V, I, W, S, T>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + InstanceSliceBackend<V, I, W, S, T>,
+  B: ?Sized + TessBackend<V, I, W, S> + InstanceSliceBackend<'a, V, I, W, S, T>,
   S: ?Sized,
   V: TessVertexData<S>,
   I: TessIndex,
@@ -1302,9 +1313,9 @@ where
   }
 }
 
-impl<B, V, I, W, S, T> DerefMut for InstancesMut<B, V, I, W, S, T>
+impl<'a, B, V, I, W, S, T> DerefMut for InstancesMut<'a, B, V, I, W, S, T>
 where
-  B: ?Sized + TessBackend<V, I, W, S> + InstanceSliceBackend<V, I, W, S, T>,
+  B: ?Sized + TessBackend<V, I, W, S> + InstanceSliceBackend<'a, V, I, W, S, T>,
   V: TessVertexData<S>,
   I: TessIndex,
   W: TessVertexData<S>,
