@@ -4,54 +4,54 @@
 //! and is a dimensional object. Textures are often associated with _images_, even though their
 //! usage in the graphics world can be much larger than that.
 //!
-//! Textures — [`Texture`] come in several flavors and the differences lie in:
+//! Textures ([`Texture`]) come in several flavors and the differences lie in:
 //!
-//! - The dimensionality: textures can be 1D, 2D, 3D, layered, cube maps, etc.
+//! - The dimensionality: textures can be 1D, 2D, 3D, layered, cube maps, etc. Dimension is encoded
+//!   via the [`Dim`] type, and indexed in the type-system with the [`Dimensionable`] and its
+//!   implementors.
 //! - The encoding: the _items_ inside textures are called _pixels_ or _texels_. Those can be
-//!   encoded in several ways.
+//!   encoded in several ways. They are represented by various types, which are implementors of the
+//!   [`Pixel`] trait.
 //! - The usage: some textures will be used as images, others will be used to pass arbitrary data
 //!   around in shaders, etc.
 //!
-//! Whatever the flavor, textures have few even not no use outside of shaders. When a shader wants
+//! Whatever the flavor, textures have few uses outside of shaders. When a shader wants
 //! to use a texture, it can do it directly, by accessing each pixel by their position inside the
 //! texture (using a normalized coordinates system) or via the use of a [`Sampler`]. A [`Sampler`],
 //! as the name implies, is an object that tells the GPU how fetching (sampling) a texture should
 //! occur. Several options there too:
 //!
-//! - The GPU can fetch a pixel without sampler. It means that you have to pass the exact
+//! - The GPU can fetch a pixel without sampling. It means that you have to pass the exact
 //!   coordinates of the pixel you want to access. This is useful when you store arbitrary
 //!   information, like UUID, velocities, etc.
 //! - The GPU can fetch a pixel with a floating-point coordinates system. How that system works
 //!   depends on the settings of [`Sampler`] you choose. For instance, you can always fetch the
 //!   _nearest_ pixel to where you sample, or you can ask the GPU to perform a linear
-//!   interpolation between all neighboring pixels, etc. [`Sampler`] allow way more than that, so
+//!   interpolation between all neighboring pixels, etc. [`Sampler`] allows way more than that, so
 //!   feel free to read their documentation.
-//!
-//! # Creating a texture
 
-use std::error;
-use std::fmt;
-use std::marker::PhantomData;
-
-use crate::backend::texture::Texture as TextureBackend;
-use crate::context::GraphicsContext;
-use crate::depth_test::DepthComparison;
-use crate::pixel::{Pixel, PixelFormat};
+use crate::{
+  backend::texture::Texture as TextureBackend,
+  context::GraphicsContext,
+  depth_stencil::Comparison,
+  pixel::{Pixel, PixelFormat},
+};
+use std::{error, fmt, marker::PhantomData};
 
 /// How to wrap texture coordinates while sampling textures?
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub enum Wrap {
-  /// If textures coordinates lay outside of *[0;1]*, they will be clamped to either *0* or *1* for
+  /// If textures coordinates lay outside of `[0;1]`, they will be clamped to either `0` or `1` for
   /// every components.
   ClampToEdge,
-  /// Textures coordinates are repeated if they lay outside of *[0;1]*. Picture this as:
+  /// Textures coordinates are repeated if they lay outside of `[0;1]`. Picture this as:
   ///
   /// ```ignore
   /// // given the frac function returning the fractional part of a floating number:
-  /// coord_ith = frac(coord_ith); // always between [0;1]
+  /// coord_ith = frac(coord_ith); // always between `[0;1]`
   /// ```
   Repeat,
-  /// Same as `Repeat` but it will alternatively repeat between *[0;1]* and *[1;0]*.
+  /// Same as `Repeat` but it will alternatively repeat between `[0;1]` and `[1;0]`.
   MirroredRepeat,
 }
 
@@ -85,7 +85,11 @@ pub enum MagFilter {
   Linear,
 }
 
-/// Reify a type into a [`Dim`].
+/// Class of [`Texture`] dimensions.
+///
+/// This trait provides a simple mapping between the implementor and the [`Dim`] type, which represents a [`Texture`]
+/// dimension. This allows to heavily type [`Texture`] so that their dimension is indexed-tracked in the type-system.
+/// This trait is then used to reify the implementors into [`Dim`] so that the runtime can work with them.
 pub trait Dimensionable {
   /// Size type of a dimension (used to caracterize dimensions’ areas).
   type Size: Copy;
@@ -94,9 +98,13 @@ pub trait Dimensionable {
   type Offset: Copy;
 
   /// Zero offset.
+  ///
+  /// Any size added with `ZERO_OFFSET` must remain the size itself.
   const ZERO_OFFSET: Self::Offset;
 
-  /// Dimension.
+  /// Reified [`Dim`].
+  ///
+  /// Implementors must ensure they map to the right variant of [`Dim`].
   fn dim() -> Dim;
 
   /// Width of the associated [`Dimensionable::Size`].
@@ -144,8 +152,14 @@ pub enum Dim {
   /// Cubemap (i.e. a cube defining 6 faces — akin to 4D).
   Cubemap,
   /// 1D array.
+  ///
+  /// This corresponds to _layered_ 1D textures, i.e. a 1D texture with an extra parameter to tap into the corresponding
+  /// layer. Using in a [`Texture`] allows to perform _layered rendering_.
   Dim1Array,
   /// 2D array.
+  ///
+  /// This corresponds to _layered_ 2D textures, i.e. a 2D texture with an extra parameter to tap into the corresponding
+  /// layer. Using in a [`Texture`] allows to perform _layered rendering_.
   Dim2Array,
 }
 
@@ -427,7 +441,7 @@ pub struct Sampler {
   /// Magnification filter.
   pub mag_filter: MagFilter,
   /// For depth textures, should we perform depth comparison and if so, how?
-  pub depth_comparison: Option<DepthComparison>,
+  pub depth_comparison: Option<Comparison>,
 }
 
 /// Default value is as following:
@@ -444,15 +458,84 @@ impl Default for Sampler {
   }
 }
 
-/// Whether mipmaps should be generated.
-#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
-pub enum GenMipmaps {
-  /// Mipmaps should be generated.
+/// Texel upload.
+///
+/// You have the choice between different options regarding mipmaps.:
+///
+/// - You can upload texels and let mipmaps being automatically created for you.
+/// - You can upload texels and disable mipmap creation.
+/// - You can upload texels by manually providing all the mipmap levels.
+#[derive(Debug)]
+pub enum TexelUpload<'a, T>
+where
+  T: ?Sized,
+{
+  /// Provide the base level and whether mipmaps should be generated.
+  BaseLevel {
+    /// Texels list to upload.
+    texels: &'a T,
+
+    /// Whether mipmap levels should be automatically created.
+    ///
+    /// Use `0` if you don’t want any mipmap.
+    mipmaps: usize,
+  },
+
+  /// Provide all the levels at once.
   ///
-  /// Mipmaps are generated when creating textures but also when uploading texels, clearing, etc.
-  Yes,
-  /// Never generate mipmaps.
-  No,
+  /// The number of elements in the outer slice represents the number of mipmaps; each inner slice represents the texels
+  /// to be uploaded to the mipmap level.
+  Levels(&'a [&'a T]),
+
+  /// Reserve only the base level and optional mipmap levels.
+  ///
+  /// This variant allows you not to pass any texel data and ask the backend to just reserve the memory for the texture.
+  /// This will allow to pass data later, and let the backend fill the data for you. The texture texels will be set in a
+  /// vendor-specific way, so you should assume that the texture will be filled with garbage.
+  Reserve {
+    /// Number of mipmap levels to allocate.
+    ///
+    /// Use `0` if you don’t want any.
+    mipmaps: usize,
+  },
+}
+
+impl<'a, T> TexelUpload<'a, T>
+where
+  T: ?Sized,
+{
+  /// Create a texel upload for the base level of a texture and let mipmap levels be automatically created.
+  pub fn base_level(texels: &'a T, mipmaps: usize) -> Self {
+    Self::BaseLevel { texels, mipmaps }
+  }
+
+  /// Create a texel upload by reserving memory and let mipmap levels be reserved as well.
+  pub fn reserve(mipmaps: usize) -> Self {
+    Self::Reserve { mipmaps }
+  }
+
+  /// Create a texel upload by manually providing all base + mipmap levels.
+  pub fn levels(texels: &'a [&'a T]) -> Self {
+    Self::Levels(texels)
+  }
+
+  /// Number of mipmaps.
+  pub fn mipmaps(&self) -> usize {
+    match self {
+      TexelUpload::BaseLevel { mipmaps, .. } => *mipmaps,
+      TexelUpload::Levels(levels) => levels.len(),
+      TexelUpload::Reserve { mipmaps } => *mipmaps,
+    }
+  }
+
+  /// Get the base level texels.
+  pub fn get_base_level(&self) -> Option<&'a T> {
+    match self {
+      TexelUpload::BaseLevel { texels, .. } => Some(*texels),
+      TexelUpload::Levels(levels) => levels.get(0).map(|base_level| *base_level),
+      TexelUpload::Reserve { .. } => None,
+    }
+  }
 }
 
 /// Errors that might happen when working with textures.
@@ -463,6 +546,7 @@ pub enum TextureError {
   ///
   /// The carried [`String`] gives the reason of the failure.
   TextureStorageCreationFailed(String),
+
   /// Not enough pixel data provided for the given area asked.
   ///
   /// You must provide at least as many pixels as expected by the area in the texture you’re
@@ -473,16 +557,19 @@ pub enum TextureError {
     /// Provided number of pixels in bytes.
     provided_bytes: usize,
   },
+
   /// Unsupported pixel format.
   ///
   /// Sometimes, some hardware might not support a given pixel format (or the format exists on
   /// the interface side but doesn’t in the implementation). That error represents such a case.
   UnsupportedPixelFormat(PixelFormat),
+
   /// Cannot retrieve texels from a texture.
   ///
   /// That error might happen on some hardware implementations if the user tries to retrieve
   /// texels from a texture that doesn’t support getting its texels retrieved.
   CannotRetrieveTexels(String),
+
   /// Failed to upload texels.
   CannotUploadTexels(String),
 }
@@ -550,7 +637,52 @@ impl fmt::Display for TextureError {
 
 impl error::Error for TextureError {}
 
-/// GPU textures.
+/// Textures.
+///
+/// Textures allow mainly two use cases:
+///
+/// - Passing data (in the form of images or regular data) to shaders.
+/// - Offscreen rendering.
+///
+/// In the former case, you will want to uplad _images_ to [`Texture`]. This can be done with the various upload functions:
+///
+/// - [`Texture::upload_part`]
+/// - [`Texture::upload`]
+/// - [`Texture::upload_part_raw`]
+/// - [`Texture::upload_raw`]
+///
+/// In the second case, a [`Texture`] can be used as part of a [`ColorSlot`] or [`DepthSlot`] of a [`Framebuffer`]. This
+/// allows to create graphics pipeline that will output into the [`Texture`], that you can use in another graphics
+/// pipeline later.
+///
+/// # Parametricity
+///
+/// Textures have three type parameters:
+///
+/// - `B`, which is the backend type. It must implement [`TextureBackend`].
+/// - `D`, the dimension of the texture. It must implement [`Dimensionable`].
+/// - `P`, the pixel type. It must implement [`Pixel`].
+///
+/// # Dimension
+///
+/// The dimension of the texture gives its flavor: 2D, 3D, cubemap, etc. You have access to a bunch of different types
+/// here, which all are implementors of the [`Dimensionable`] trait.
+///
+/// # Pixel format
+///
+/// The internal representation of the [`Texture`] will be derived from the `P` type, which is the [`Pixel`] type. Lots
+/// of types are available, but you have to know that depending on the use you want to make of the texture, not all
+/// types are compatible. For instance, if you access a [`Texture`] as part of a [`ColorSlot`] of a [`Framebuffer`], the
+/// [`Pixel`] type must be [`RenderablePixel`]. The compiler will always tells you if you are trying to use a pixel type
+/// that is not compatible, but you should check the implementors of [`Pixel`], [`RenderablePixel`] and [`DepthPixel`]
+/// before starting using them.
+///
+/// [`Framebuffer`]: crate::framebuffer::Framebuffer
+/// [`ColorSlot`]: crate::backend::color_slot::ColorSlot
+/// [`DepthSlot`]: crate::backend::depth_slot::DepthSlot
+/// [`RenderablePixel`]: crate::pixel::RenderablePixel
+/// [`DepthPixel`]: crate::pixel::DepthPixel
+/// [`TextureBackend`]: crate::backend::texture::Texture
 pub struct Texture<B, D, P>
 where
   B: ?Sized + TextureBackend<D, P>,
@@ -568,54 +700,16 @@ where
   D: Dimensionable,
   P: Pixel,
 {
-  /// Create a new [`Texture`] by reserving space for texels.
-  ///
-  /// `size` is the wished size of the [`Texture`].
-  ///
-  /// `mipmaps` is the number of extra mipmaps to allocate with the texture. `0` means that the
-  /// texture will only be made of a _base level_.
-  ///
-  /// `sampler` is a [`Sampler`] object that will be used when sampling the texture from inside a
-  /// shader, for instance.
-  ///
-  /// # Notes
-  ///
-  /// Feel free to have a look at the documentation of [`GraphicsContext::new_texture_no_texels`] for a
-  /// simpler interface.
-  pub(crate) fn new_no_texels<C>(
-    ctx: &mut C,
-    size: D::Size,
-    mipmaps: usize,
-    sampler: Sampler,
-  ) -> Result<Self, TextureError>
-  where
-    C: GraphicsContext<Backend = B>,
-  {
-    unsafe {
-      ctx
-        .backend()
-        .new_texture(size, mipmaps, sampler)
-        .map(|repr| Texture {
-          repr,
-          size,
-          _phantom: PhantomData,
-        })
-    }
-  }
-
   /// Create a new [`Texture`].
   ///
-  /// `size` is the wished size of the [`Texture`].
-  ///
-  /// `mipmaps` is the number of extra mipmaps to allocate with the texture. `0` means that the
-  /// texture will only be made of a _base level_.
+  /// `size` is the desired size of the [`Texture`].
   ///
   /// `sampler` is a [`Sampler`] object that will be used when sampling the texture from inside a
   /// shader, for instance.
   ///
   /// `gen_mipmaps` determines whether mipmaps should be generated automatically.
   ///
-  /// `texels` is a slice of raw texels to put into the texture store.
+  /// `texels` is a [`TexelUpload`] of texels to put into the texture store.
   ///
   /// # Notes
   ///
@@ -624,41 +718,32 @@ where
   pub fn new<C>(
     ctx: &mut C,
     size: D::Size,
-    mipmaps: usize,
     sampler: Sampler,
-    gen_mipmaps: GenMipmaps,
-    texels: &[P::Encoding],
+    texels: TexelUpload<[P::Encoding]>,
   ) -> Result<Self, TextureError>
   where
     C: GraphicsContext<Backend = B>,
   {
-    let mut tex = unsafe {
+    unsafe {
       ctx
         .backend()
-        .new_texture(size, mipmaps, sampler)
+        .new_texture(size, sampler, texels)
         .map(|repr| Texture {
           repr,
           size,
           _phantom: PhantomData,
-        })?
-    };
-    tex.upload(gen_mipmaps, texels)?;
-    Ok(tex)
+        })
+    }
   }
 
   /// Create a new [`Texture`] with raw texels.
   ///
   /// `size` is the wished size of the [`Texture`].
   ///
-  /// `mipmaps` is the number of extra mipmaps to allocate with the texture. `0` means that the
-  /// texture will only be made of a _base level_.
-  ///
   /// `sampler` is a [`Sampler`] object that will be used when sampling the texture from inside a
   /// shader, for instance.
   ///
-  /// `gen_mipmaps` determines whether mipmaps should be generated automatically.
-  ///
-  /// `texels` is a slice of raw texels to put into the texture store.
+  /// `texels` is a [`TexelUpload`] of raw texels to put into the texture store.
   ///
   /// # Notes
   ///
@@ -667,26 +752,22 @@ where
   pub fn new_raw<C>(
     ctx: &mut C,
     size: D::Size,
-    mipmaps: usize,
     sampler: Sampler,
-    gen_mipmaps: GenMipmaps,
-    texels: &[P::RawEncoding],
+    texels: TexelUpload<[P::RawEncoding]>,
   ) -> Result<Self, TextureError>
   where
     C: GraphicsContext<Backend = B>,
   {
-    let mut tex = unsafe {
+    unsafe {
       ctx
         .backend()
-        .new_texture(size, mipmaps, sampler)
+        .new_texture_raw(size, sampler, texels)
         .map(|repr| Texture {
           repr,
           size,
           _phantom: PhantomData,
-        })?
-    };
-    tex.upload_raw(gen_mipmaps, texels)?;
-    Ok(tex)
+        })
+    }
   }
 
   /// Return the number of mipmaps.
@@ -706,13 +787,10 @@ where
   pub fn resize(
     &mut self,
     size: D::Size,
-    mipmaps: usize,
-    gen_mipmaps: GenMipmaps,
-    texels: &[P::Encoding],
+    texels: TexelUpload<[P::Encoding]>,
   ) -> Result<(), TextureError> {
     self.size = size;
-    unsafe { B::resize(&mut self.repr, size, mipmaps) }?;
-    self.upload(gen_mipmaps, texels)
+    unsafe { B::resize(&mut self.repr, size, texels) }
   }
 
   /// Resize the texture by providing a new size and raw texels by reusing its GPU resources.
@@ -722,76 +800,42 @@ where
   pub fn resize_raw(
     &mut self,
     size: D::Size,
-    mipmaps: usize,
-    gen_mipmaps: GenMipmaps,
-    texels: &[P::RawEncoding],
+    texels: TexelUpload<[P::RawEncoding]>,
   ) -> Result<(), TextureError> {
     self.size = size;
-    unsafe { B::resize(&mut self.repr, size, mipmaps) }?;
-    self.upload_raw(gen_mipmaps, texels)
-  }
-
-  /// Clear the texture with a single pixel value.
-  ///
-  /// This function will assign the input pixel value to all the pixels in the rectangle described
-  /// by `size` and `offset` in the texture.
-  pub fn clear_part(
-    &mut self,
-    gen_mipmaps: GenMipmaps,
-    offset: D::Offset,
-    size: D::Size,
-    pixel: P::Encoding,
-  ) -> Result<(), TextureError> {
-    unsafe { B::clear_part(&mut self.repr, gen_mipmaps, offset, size, pixel) }
-  }
-
-  /// Clear the texture with a single pixel value.
-  ///
-  /// This function will assign the input pixel value to all the pixels in the texture.
-  pub fn clear(&mut self, gen_mipmaps: GenMipmaps, pixel: P::Encoding) -> Result<(), TextureError> {
-    unsafe { B::clear(&mut self.repr, gen_mipmaps, self.size, pixel) }
+    unsafe { B::resize_raw(&mut self.repr, size, texels) }
   }
 
   /// Upload pixels to a region of the texture described by the rectangle made with `size` and
   /// `offset`.
   pub fn upload_part(
     &mut self,
-    gen_mipmaps: GenMipmaps,
     offset: D::Offset,
     size: D::Size,
-    texels: &[P::Encoding],
+    texels: TexelUpload<[P::Encoding]>,
   ) -> Result<(), TextureError> {
-    unsafe { B::upload_part(&mut self.repr, gen_mipmaps, offset, size, texels) }
+    unsafe { B::upload_part(&mut self.repr, offset, size, texels) }
   }
 
   /// Upload pixels to the whole texture.
-  pub fn upload(
-    &mut self,
-    gen_mipmaps: GenMipmaps,
-    texels: &[P::Encoding],
-  ) -> Result<(), TextureError> {
-    unsafe { B::upload(&mut self.repr, gen_mipmaps, self.size, texels) }
+  pub fn upload(&mut self, texels: TexelUpload<[P::Encoding]>) -> Result<(), TextureError> {
+    unsafe { B::upload(&mut self.repr, self.size, texels) }
   }
 
   /// Upload raw data to a region of the texture described by the rectangle made with `size` and
   /// `offset`.
   pub fn upload_part_raw(
     &mut self,
-    gen_mipmaps: GenMipmaps,
     offset: D::Offset,
     size: D::Size,
-    texels: &[P::RawEncoding],
+    texels: TexelUpload<[P::RawEncoding]>,
   ) -> Result<(), TextureError> {
-    unsafe { B::upload_part_raw(&mut self.repr, gen_mipmaps, offset, size, texels) }
+    unsafe { B::upload_part_raw(&mut self.repr, offset, size, texels) }
   }
 
   /// Upload raw data to the whole texture.
-  pub fn upload_raw(
-    &mut self,
-    gen_mipmaps: GenMipmaps,
-    texels: &[P::RawEncoding],
-  ) -> Result<(), TextureError> {
-    unsafe { B::upload_raw(&mut self.repr, gen_mipmaps, self.size, texels) }
+  pub fn upload_raw(&mut self, texels: TexelUpload<[P::RawEncoding]>) -> Result<(), TextureError> {
+    unsafe { B::upload_raw(&mut self.repr, self.size, texels) }
   }
 
   /// Get a copy of all the pixels from the texture.
